/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package com.mousman.as3.agal.camera
{
	import com.mousman.as3.agal.mesh.IMMesh;
	import flash.display.Stage;
	import flash.geom.Matrix3D;
	import flash.geom.Orientation3D;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	/**
	 */
	public class OrbitalCamera implements IMCamera 
	{
		private const RAD_TO_DEG:Number = 180 / Math.PI;
		
		private var _targetObjectMatrix:Matrix3D;
		private var _stage:Stage;
		private var _radius:Number;
		
		private var _matrix:Matrix3D;
		
		private var _mouseX:Number;
		private var _mouseY:Number;		
		private var _mouseIsDown:Boolean;		
		
		private var _currentAxis:Vector3D;
		private var _currentAngle:Number;
		
		private var _savePos:Point;
		private var _saveFromVector:Vector3D;
		
		public function OrbitalCamera(targetObjectMatrix:Matrix3D, stage:Stage, radius:Number = 10) 
		{
			_targetObjectMatrix = new Matrix3D();
			_targetObjectMatrix.position = targetObjectMatrix.position;
			_stage = stage;
			_radius = radius;
			
			init();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function init():void
		{
			_matrix = new Matrix3D();
			_currentAxis = new Vector3D(0,1,0);
			_currentAngle = 0;
		
			_matrix.appendTranslation(0, 0, -radius);			
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseDown():void
		{
			_mouseIsDown = true;
			
			var x:Number = (_mouseX -_stage.stageWidth/2) *2 / _stage.stageWidth;
			var y:Number  = (_stage.stageHeight / 2 - _mouseY) * 2 / _stage.stageHeight;
			_savePos = new Point(x, y);
			_saveFromVector = coordinate2DToSphere(x, y);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function update():void
		{
			if (!_mouseIsDown) return;
			var x:Number = (_mouseX -_stage.stageWidth/2) *2 / _stage.stageWidth;
			var y:Number  = (_stage.stageHeight / 2 - _mouseY) * 2 / _stage.stageHeight;		
			// waiting for delta to be significatnt
			var deltaX:Number = Math.abs(_savePos.x - x);
			var deltaY:Number = Math.abs(_savePos.y - y);
			if ( deltaX < 0.01 && deltaY < 0.01) return; 
			
			var newVector:Vector3D = coordinate2DToSphere(x - _savePos.x, y - _savePos.y);
			
			drag(newVector);
			_saveFromVector = newVector;
			_savePos = new Point(x, y);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		private function drag(newVector:Vector3D):void
		{
			var fromVector:Vector3D = coordinate2DToSphere(0, 0);
			
			var resultVector:Vector3D = fromVector.crossProduct(newVector);
			var axis:Vector3D = resultVector;
			var angle:Number  = Vector3D.angleBetween(fromVector, newVector);
			angle = RAD_TO_DEG * angle;			
			
			if(axis.length != 1)
				axis.normalize();
			
			_currentAxis = axis;
			_currentAngle = angle;
			_matrix.appendRotation(angle, axis, _targetObjectMatrix.position);
			
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseUp():void
		{
			_mouseIsDown = false;
			_saveFromVector = coordinate2DToSphere(_savePos.x, _savePos.y);
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseMove(mouseX:Number, mouseY:Number):void
		{
			_mouseX = mouseX;
			_mouseY = mouseY;
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function coordinate2DToSphere(x:Number, y:Number):Vector3D
		{
			var targetVector:Vector3D = new Vector3D();
			
			targetVector.x = x;
			targetVector.y = y;
			targetVector.z = 0;

			if(targetVector.lengthSquared > 1){
				targetVector.normalize();
			}
			else{
				targetVector.z = Math.sqrt(1 - targetVector.lengthSquared);
			}
			
			targetVector = _matrix.deltaTransformVector(targetVector);
			
			return targetVector;
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function mouseWheel(delta:Number):void
		{
			_radius += delta ;
			_matrix.identity();
			_matrix.appendTranslation(0, 0, -_radius);
			_matrix.appendRotation(_currentAngle, _currentAxis, _targetObjectMatrix.position);
		}
			
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------
		public function get matrix():Matrix3D {return _matrix;}
		public function set matrix(value:Matrix3D):void { _matrix = value; }
		
		public function get position():Vector3D {return _matrix.position;}
		
		public function get radius():Number {return _radius;}
		public function set radius(value:Number):void {
			_radius = value;
			_matrix.identity();
			_matrix.appendTranslation(0, 0, -_radius);
			_matrix.appendRotation(_currentAngle, _currentAxis, _targetObjectMatrix.position);			
		}
	}

}