/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.light 
{
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	/**
	 * ...
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MLight implements IMLight 
	{
		private var _diffuseColor:uint;
		private var _specularColor:uint;
		private var _position:Vector3D;
		private var _rotation:Vector3D;
		private var _matrix:Matrix3D;
		private var _diffuseIntensity:Number;
		private var _specularIntensity:Number;
		
		public function MLight(diffuseColor:uint,diffuseIntensity:Number=0.5,specularColor:uint=0xffffff,specularIntensity:Number=0.5) 
		{
			_diffuseColor = diffuseColor;
			_diffuseIntensity = diffuseIntensity;
			_specularColor = specularColor;
			_specularIntensity = specularIntensity;
			
			if ( !_matrix) _matrix = new Matrix3D();
			if ( !_rotation) _rotation = new Vector3D();
			if ( !_position) _position = new Vector3D();
			
			_matrix.identity();
			
			_matrix.appendRotation(_rotation.x, Vector3D.X_AXIS);
			_matrix.appendRotation(_rotation.y, Vector3D.Y_AXIS);
			_matrix.appendRotation(_rotation.z, Vector3D.Z_AXIS);
			
			_matrix.appendTranslation(_position.x, _position.y, _position.z);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function updateMatrix():void {
			if ( !_matrix) _matrix = new Matrix3D();
			if ( !_rotation) _rotation = new Vector3D();
			if ( !_position) _position = new Vector3D();
			
			_matrix.identity();
			
			_matrix.appendRotation(_rotation.x, Vector3D.X_AXIS);
			_matrix.appendRotation(_rotation.y, Vector3D.Y_AXIS);
			_matrix.appendRotation(_rotation.z, Vector3D.Z_AXIS);			
			
			_matrix.appendTranslation(_position.x, _position.y, _position.z);	
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function get position():Vector3D {return _position;}
		public function set position(value:Vector3D):void {	_position = value;}
		
		public function get rotation():Vector3D {return _rotation;}
		public function setRotation(rx:Number, ry:Number, rz:Number):void { _rotation.x = rx; _rotation.y = ry;_rotation.z = rz; updateMatrix(); }
		
		public function get matrix():Matrix3D {return _matrix;}
		public function set matrix(value:Matrix3D):void {_matrix = value;}

		
		public function get diffuseColor():uint {return _diffuseColor;}
		public function set diffuseColor(value:uint):void {_diffuseColor = value;}
		
		public function get specularColor():uint {return _specularColor;}
		public function set specularColor(value:uint):void {_specularColor = value;}
		
		public function get diffuseIntensity():Number {return _diffuseIntensity;}
		public function set diffuseIntensity(value:Number):void {_diffuseIntensity = value;}
		
		public function get specularIntensity():Number {return _specularIntensity;}
		public function set specularIntensity(value:Number):void {_specularIntensity = value;}
	}

}