/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.objs 
{
	import com.mousman.as3.structure.garbage.IDestroyable;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.VertexBuffer3D;
	import flash.utils.ByteArray;
	
	/**
	 * MPGBuffersExport can be a part of MPolygoneGroup3D 
	 * <p>It is used with the Exporter class</p>
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MPGBuffersExport implements IDestroyable
	{
		
		/**
		 * @private
		 */		
		private var _vertexBuffer:VertexBuffer3D;
		/**
		 * @private
		 */		
		private var _indexBuffer:IndexBuffer3D;
		/**
		 * @private
		 */		
		private var _id:String;
		/**
		 * @private
		 */		
		private var _vertexBufferBA:ByteArray;
		/**
		 * @private
		 */		
		private var _indexBufferV:Vector.<uint>;
		/**
		 * @private
		 */		
		private var _dataPerVertex:int;
		/**
		 * @private
		 */		
		private var _nbVertices:int;
		/**
		 * @private
		 */		
		private var _nbIndices:int;
				
		/**
		* Constructor. Creates a new MPGBuffersExport instance.
		*
		* @param id : String.<p>Id given to an instance of MPGBuffersExport</p>
		*/			
		public function MPGBuffersExport(id:String) 
		{
			_id = id;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		public function destroy():void {
			_vertexBuffer.dispose();
			_vertexBuffer = null;
			_indexBuffer.dispose();
			_indexBuffer = null;
			_vertexBufferBA.clear();
			_vertexBufferBA = null;
			_indexBufferV.length = 0;
			_indexBufferV = null;
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------	
		/**
		* the VertexBuffer3D contained in a instance of MPGBuffersExport
		*/ 			
		public function get vertexBuffer():VertexBuffer3D {return _vertexBuffer;}
		public function set vertexBuffer(value:VertexBuffer3D):void {_vertexBuffer = value;}
		
		/**
		*  the IndexBuffer3D contained in a instance of MPolygonGroupBuffers
		*/ 			
		public function get indexBuffer():IndexBuffer3D {return _indexBuffer;}
		public function set indexBuffer(value:IndexBuffer3D):void {_indexBuffer = value;}
		
		/**
		* id of a polygon group MPolygonGroupBuffers
		*/ 			
		public function get id():String {return _id;}
		
		
		/**
		* ByteArray used for a VertexBuffer3D
		*/ 			
		public function get vertexBufferBA():ByteArray {return _vertexBufferBA;}
		public function set vertexBufferBA(value:ByteArray):void {_vertexBufferBA = value;}
		
		/**
		* Vector of indices used for an IndexBuffer3D
		*/ 			
		public function get indexBufferV():Vector.<uint> {return _indexBufferV;}
		public function set indexBufferV(value:Vector.<uint>):void {_indexBufferV = value;}
		
		
		/**
		* data per vertex 
		*/ 			
		public function get dataPerVertex():int {return _dataPerVertex;}
		public function set dataPerVertex(value:int):void {	_dataPerVertex = value;}
		
		/**
		* number of vertices in an instance of a MPGBuffersExport instance
		*/ 			
		public function get nbVertices():int {return _nbVertices;}
		public function set nbVertices(value:int):void {_nbVertices = value;}
		
		/**
		* number of indices in an instance of a MPGBuffersExport instance
		*/ 				
		public function get nbIndices():int {return _nbIndices;}
		public function set nbIndices(value:int):void {_nbIndices = value;}
	}

}