/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.parser.shaders
{
	import com.mousman.as3.agal.parser.verticeswriter.IMBytesArrayWriter;
	import com.mousman.as3.agal.shader.IMShader;
	import flash.display3D.Context3D;
	import flash.display3D.Program3D;
	/**
	 * A ShaderSet has in charge to linked the fragment shader, the vertex shader and the writer used to create the VertexBuffer and the IndexBuffer
	 * 
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class ShaderSet 
	{
		
		private var _shader:IMShader;
		private var _writer:IMBytesArrayWriter;
		private var _id:String;
		
		/**
		* Constructor. Creates a new ShaderSet instance.
		*
		* @param id : String.<p>Id of a ShaderSet instance</p>
		* @param shader : IMShader.<p>an IMShader instance</p>
		* @param writer : IMBytesArrayWriter.<p>a IMBytesArrayWriter instance</p>
		*/			
		public function ShaderSet(id:String,shader:IMShader,writer:IMBytesArrayWriter) 
		{
			_id = id;
			_shader = shader;
			_writer = writer;
		}
		//----------------------------------------------------------------------------------------------		
		/**
		* Assemble and upload the fragment and vertex shaders
		*
		* @param context3D : Context3D.<p>the context with wich the shaders will be assembled and uploaded</p>
		*/			
		//----------------------------------------------------------------------------------------------		
		public function assembleNupload(context3D:Context3D):void {
			 _shader.assemble(context3D);
			 _shader.upload();
		}
		//----------------------------------------------------------------------------------------------	
		/**
		* Assemble the fragment and vertex shaders
		*
		* @param context3D : Context3D.<p>the context with wich the shaders will be assembled and uploaded</p>
		*/		
		//----------------------------------------------------------------------------------------------		
		public function assemble(context3D:Context3D):void {
			 _shader.assemble(context3D);
		}
		//----------------------------------------------------------------------------------------------	
		/**
		* Upload the fragment and vertex shaders
		*
		* @param context3D : Context3D.<p>the context with wich the shaders will be assembled and uploaded</p>
		*/		
		//----------------------------------------------------------------------------------------------		
		public function upload():void {
			 _shader.upload();
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function destroy():void {
			_shader.destroy();
			_shader = null;
			_writer.destroy();
			_writer = null;
		}			
		//----------------------------------------------------------------------------------------------		
		/**
		* IMShader instance
		*/ 	
		public function get shader():IMShader {	return _shader;}
		public function set shader(value:IMShader):void {_shader = value;}
		
		/**
		* IMBytesArrayWriter instance
		*/ 	
		public function get writer():IMBytesArrayWriter {return _writer;}
		public function set writer(value:IMBytesArrayWriter):void { _writer = value; }
		
		/**
		* Id of a ShaderSet instance
		*/ 		
		public function get id():String {return _id;}
		
		/**
		* program of the shader
		*/ 		
		public function get program():Program3D {return _shader.shaderProgram;}
	}

}