/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.shader
{
	import com.adobe.utils.AGALMiniAssembler;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DProgramType;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.Program3D;
	import flash.display3D.VertexBuffer3D;
	/**
	 * ...
	 * @author Moussa Dembélé (www.mousman.com)
	 * va0 : position x,y,z
	 * va1 : color r,g,b
	 * 
	 * vc0 : scene matrix
	 */
	
	public class MBasicShader implements IMShader
	{
		private var _vertexProgram:String;
		private var _fragmentProgram:String;
		private var _shaderProgram:Program3D;
		private var _vertexShaderAssembler:AGALMiniAssembler;
		private var _fragmentShaderAssembler:AGALMiniAssembler;			
		private var _dataPerVertex:int = 7;
		
		public function MBasicShader() 
		{
			createVertexProgram();
			createFragmentProgram();			
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function createVertexProgram():void {
			_vertexProgram =	
				"m44 op        ,va0  ,vc0       \n" + // transform vertex x,y,z
				"mov v0        ,va1             \n";  // move vertex r,g,b to fragment shader
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function createFragmentProgram():void {
			_fragmentProgram = "" +
				"mov oc,      v0                  \n";  // output						
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function setActiveVertextBuffer(context3D:Context3D, vertexBuffer:VertexBuffer3D):void {
			// XYZ
			context3D.setVertexBufferAt(0, vertexBuffer, 0, Context3DVertexBufferFormat.FLOAT_3);
			// colors
			context3D.setVertexBufferAt(1, vertexBuffer, 3, Context3DVertexBufferFormat.BYTES_4);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function releaseActiveVertextBuffer(context3D:Context3D):void {
			// XYZ
			context3D.setVertexBufferAt(0, null);
			// colors
			context3D.setVertexBufferAt(1, null);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function assembleNupload(context3D:Context3D):void {
			 assemble(context3D);
			 upload();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function assemble(context3D:Context3D):void {
			_vertexShaderAssembler = new AGALMiniAssembler();
			_vertexShaderAssembler.assemble(Context3DProgramType.VERTEX, _vertexProgram);
			
			_fragmentShaderAssembler = new AGALMiniAssembler();
			_fragmentShaderAssembler.assemble(Context3DProgramType.FRAGMENT, _fragmentProgram);
			_shaderProgram = context3D.createProgram();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function upload():void {
			_shaderProgram.upload(_vertexShaderAssembler.agalcode, _fragmentShaderAssembler.agalcode);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function destroy():void {
			_shaderProgram.dispose();
		}		
		//----------------------------------------------------------------------------------------------		
		public function get vertexProgram():String{return _vertexProgram;}
		public function get fragmentProgram():String { return _fragmentProgram; }
		public function get shaderProgram():Program3D {return _shaderProgram;}
		
		public function get dataPerVertex():int {return _dataPerVertex;}
	}

}