/*Copyright (c) 2012 Moussa Dembélé (www.mousman.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.mousman.as3.agal.shader
{
	import com.adobe.utils.AGALMiniAssembler;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DProgramType;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.Program3D;
	import flash.display3D.VertexBuffer3D;
	/**
	 * ...
	 * @author Moussa Dembélé (www.mousman.com)
	 */
	public class MPhong2LightsTextureShader implements IMShader
	{
		
		private var _vertexTextureProgram:String;
		private var _fragmentTextureProgram:String;
		private var _shaderProgram:Program3D;
		private var _vertexShaderAssembler:AGALMiniAssembler;
		private var _fragmentShaderAssembler:AGALMiniAssembler;		
		private var _dataPerVertex:int = 9;
		
		public function MPhong2LightsTextureShader() 
		{
			createVertexTextureProgram();
			createFragmentTextureProgram();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function createVertexTextureProgram():void {
			_vertexTextureProgram =	
				"m44 vt0,       va0,  vc0        \n" + // transform vertex x,y,z
				"mov op,        vt0              \n" + // transform vertex x,y,z
				"mov v0,        va1              \n" + // move vertex r,g,b to fragment shader
				"m44 v1,        va2,  vc8        \n" + // v1, interpolate normal position in the scene
				"m44 v2,        va0,  vc4        \n" + // v2, interpolate object position in the scene					
				"mov v3,        va3              \n";  //move vertex u,v to fragment shader
		}		
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		private function createFragmentTextureProgram():void 
		{
			_fragmentTextureProgram = "" +
				"sub ft0,       fc2,      v2  \n" + // get dir from the fragment to the light
				"nrm ft0.xyz,   ft0.xyz       \n" +  // normalize the light direction	
				"mov ft0.w,     fc2.w         \n" +  
				
				"sub ft1       ,fc7,     v2   \n" + // get dir from the fragment to camera
				"nrm ft1.xyz   ,ft1           \n" +
				"mov ft1.w     ,fc2.w         \n" + 	
				
				"mov ft2       ,v1            \n" + //normals
				"nrm ft2.xyz   ,ft2           \n" +
				"mov ft2.w     ,fc2.w         \n" + // 
				
				"tex ft3, v3, fs0<2d, linear, nomip> \n" +
				
				 //calculate diffuse term - ft4
				"dp3 ft4,       ft2,       ft0       \n" + // find projection of direction to light on normal - ft4
				"mov ft5,       ft4                  \n" + // save for specular light
				"sat ft4,       ft4        	   	     \n" + // ignore negative values
				"mul ft4.xyz,   ft4.xyz,   fc1.w     \n" + // multiply projection with light's diffuse amount
				"mul ft4.xyz,   ft4.xyz    fc1.xyz   \n" + // multiply projection of direction to light on normal with light's diffuse color
				"mul ft4.xyz,   ft4.xyz,   ft3       \n" + // multiply by material's diffuse color

				 //calculate specular term - ft5
				 //reflected light
				"mul ft5,      ft5,        ft2       \n" + 
				"add ft5,      ft5,        ft5       \n" + 
				"sub ft5,      ft5,        ft0       \n" + 
				 
				"dp3 ft5        ft5,       ft1       \n" + 
				"sat ft5,       ft5                  \n" + // 
				"pow ft5,       ft5,       fc9.w     \n" + // apply concentration (gloss)
				"mul ft5.xyz,   ft5.xyz,   fc3.xyz   \n" + // multiply with light's specular color
				"mul ft5.xyz,   ft5.xyz,   fc3.w     \n" + // multiply with light's specular amount
				"mul ft5.xyz,   ft5.xyz,   fc9.xyz   \n" + // multiply with material's specular color
				
				"add ft6.xyz, ft5.xyz, ft4.xyz    \n" + // combine diffuse and specular terms
				
				// 2nd light
				"sub ft0,       fc5,    v2     \n" + // get dir from the fragment to the light 2
				"nrm ft0.xyz,   ft0.xyz        \n" +  // normalize the light direction	
				"mov ft0.w ,    fc2.w          \n" +  
				
				 //calculate diffuse term - ft4
				"dp3 ft4,       ft2,       ft0       \n" + // find projection of direction to light on normal - ft4
				"mov ft5,       ft4                  \n" + // save for specular light
				"sat ft4,       ft4        	   	     \n" + // ignore negative values
				"mul ft4.xyz,   ft4.xyz,   fc1.w     \n" + // multiply projection with light's diffuse amount
				"mul ft4.xyz,   ft4.xyz    fc1.xyz   \n" + // multiply projection of direction to light on normal with light's diffuse color
				"mul ft4.xyz,   ft4.xyz,   ft3       \n" + // multiply by material's diffuse color
				
				 //calculate specular term - ft5
				 //reflected light
				"mul ft5,      ft5,        ft2       \n" + 
				"add ft5,      ft5,        ft5       \n" + 
				"sub ft5,      ft5,        ft0       \n" + 
				 
				"dp3 ft5        ft5,       ft1       \n" + 
				"sat ft5,       ft5                  \n" + // 
				"pow ft5,       ft5,       fc9.w     \n" + // apply concentration (gloss)
				"mul ft5.xyz,   ft5.xyz,   fc6.xyz   \n" + // multiply with light's specular color
				"mul ft5.xyz,   ft5.xyz,   fc6.w     \n" + // multiply with light's specular amount
				"mul ft5.xyz,   ft5.xyz,   fc9.xyz   \n" + // multiply with material's specular color
				
				 //output - ft6
				"add ft6.xyz, ft6.xyz, ft4.xyz    \n" + // combine diffuse and specular terms
				"add ft6.xyz, ft6.xyz, ft5.xyz    \n" + // combine diffuse and specular terms
				
				"mov ft7, fc0                      \n" + // ambiant light
				"mul ft7.xyz, ft7.xyz, fc8.xyz     \n" + // multiply   ambiant light color with  material's ambiant color
				"mul ft7.xyz, ft7.xyz, fc0.w       \n" + // multiply with ambiant light amount
				
				"add ft7.xyz, ft7.xyz, ft6.xyz     \n" + // add diffuse and specular terms
				
				"mov ft7.w,      v0.w             \n" + // alpha value
				"mov oc,         ft7              \n";  // output	
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function setActiveVertextBuffer(context3D:Context3D,vertexBuffer:VertexBuffer3D):void {
			// XYZ
			context3D.setVertexBufferAt(0, vertexBuffer, 0, Context3DVertexBufferFormat.FLOAT_3);
			// colors
			context3D.setVertexBufferAt(1, vertexBuffer, 3, Context3DVertexBufferFormat.BYTES_4);
			// normales
			context3D.setVertexBufferAt(2, vertexBuffer, 4, Context3DVertexBufferFormat.FLOAT_3);
			// uvs
			context3D.setVertexBufferAt(3, vertexBuffer, 7, Context3DVertexBufferFormat.FLOAT_2);
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function releaseActiveVertextBuffer(context3D:Context3D):void {
			// XYZ
			context3D.setVertexBufferAt(0, null);
			// colors
			context3D.setVertexBufferAt(1, null);
			// normales
			context3D.setVertexBufferAt(2, null);
			// uvs
			context3D.setVertexBufferAt(3, null);
		}				
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function assembleNupload(context3D:Context3D):void {
			 assemble(context3D);
			 upload();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function assemble(context3D:Context3D):void {
			_vertexShaderAssembler = new AGALMiniAssembler();
			_vertexShaderAssembler.assemble(Context3DProgramType.VERTEX, _vertexTextureProgram);
			
			_fragmentShaderAssembler = new AGALMiniAssembler();
			_fragmentShaderAssembler.assemble(Context3DProgramType.FRAGMENT, _fragmentTextureProgram);
			_shaderProgram = context3D.createProgram();
		}
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function upload():void {
			_shaderProgram.upload(_vertexShaderAssembler.agalcode, _fragmentShaderAssembler.agalcode);
		}	
		//----------------------------------------------------------------------------------------------		
		//----------------------------------------------------------------------------------------------		
		public function destroy():void {
			_shaderProgram.dispose();
		}		
		//----------------------------------------------------------------------------------------------		
		public function get vertexProgram():String{return _vertexTextureProgram;}
		public function get fragmentProgram():String { return _fragmentTextureProgram; }	
		public function get shaderProgram():Program3D {return _shaderProgram;}
		
		public function get dataPerVertex():int {return _dataPerVertex;}
	}

}